package com.mcwl.communityCenter.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.mcwl.common.core.domain.AjaxResult;
import com.mcwl.common.core.domain.entity.SysUser;
import com.mcwl.common.utils.SecurityUtils;
import com.mcwl.communityCenter.domain.Publish;
import com.mcwl.communityCenter.domain.dto.CommentRes;
import com.mcwl.communityCenter.domain.vo.PublicModelCommentVo;
import com.mcwl.communityCenter.domain.PublishComment;
import com.mcwl.communityCenter.domain.PublishCommentLike;
import com.mcwl.communityCenter.mapper.PublishCommentLikeMapper;
import com.mcwl.communityCenter.mapper.PublishCommentMapper;
import com.mcwl.communityCenter.mapper.PublishMapper;
import com.mcwl.communityCenter.service.PublishCommentService;
import com.mcwl.system.service.ISysUserService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * 评论区业务层
 *
 * @Author：ChenYan
 * @Project：mcwl-ai
 * @Package：com.mcwl.communityCenter.service.impl
 * @Filename：PublishCommissionServiceImpl
 * @Description TODO
 * @Date：2025/1/17 14:28
 */
@Service
@RequiredArgsConstructor
public class PublishCommentServiceImpl extends ServiceImpl<PublishCommentMapper, PublishComment> implements PublishCommentService {

    private final PublishCommentMapper publishCommentMapper;

    private final PublishCommentLikeMapper likeMapper;

    private final ISysUserService sysUserService;

    private final PublishMapper publishMapper;

    /**
     * 点赞或取消点赞评论
     *
     * @param userId    用户ID
     * @param commentId 评论ID
     * @return 更新后的点赞数
     */
    @Override
    @Transactional
    public int toggleLike(Long userId, Long commentId) {
        // 检查评论是否存在
        PublishComment comment = this.getById(commentId);
        if (comment == null) {
            throw new RuntimeException("Comment not found");
        }

        // 检查用户是否已经点赞
        PublishCommentLike like = likeMapper.selectOne(
                new LambdaQueryWrapper<PublishCommentLike>()
                        .eq(PublishCommentLike::getUserId, userId)
                        .eq(PublishCommentLike::getPublishCommentId, commentId));

        if (like == null) { // 如果没有找到点赞记录，则进行点赞
            like = new PublishCommentLike();
            like.setUserId(userId);
            like.setCommunityId(1L); // 假设有一个默认的社区ID
            like.setPublishCommentId(commentId);
            likeMapper.insert(like);

            // 更新评论的点赞数
            comment.setLikeNum(comment.getLikeNum() == null ? 1 : comment.getLikeNum() + 1);
        } else { // 如果找到了点赞记录，则取消点赞
            likeMapper.deleteById(like.getId());

            // 更新评论的点赞数
            comment.setLikeNum(Math.max(0, comment.getLikeNum() - 1));
        }

        // 保存更新后的评论信息
        this.updateById(comment);

        return comment.getLikeNum();
    }

    @Override
    public List<PublicModelCommentVo> getComment(PublishComment publishComment) {
        List<PublicModelCommentVo> CommentVoList = new ArrayList<>();

        // 查询所有父评论
        LambdaQueryWrapper<PublishComment> lqw = new LambdaQueryWrapper<>();
        lqw.eq(PublishComment::getTenantId, publishComment.getTenantId())
                .isNull(PublishComment::getParentId)
                .orderByDesc(PublishComment::getCreateTime);
        // 添加父评论
        List<PublishComment> modelCommentList = publishCommentMapper.selectList(lqw);
        for (PublishComment Comment : modelCommentList) {
            PublicModelCommentVo CommentVo = getModelCommentVo(Comment);
            CommentVoList.add(CommentVo);
        }

        return CommentVoList;
    }

    @Override
    public AjaxResult addComment(CommentRes commentRes) {
        Long tenantId = commentRes.getTenantId();
        Long communityId = commentRes.getCommunityId();
        Long operatorId = commentRes.getOperatorId();

        Publish publish = publishMapper.selectByIdAndTenantIdAndCommunityId(operatorId, tenantId, communityId);
        if (Objects.isNull(publish)) {
            return AjaxResult.error("评论失败，该内容不存在");
        }

        PublishComment publishComment = new PublishComment();
        BeanUtil.copyProperties(commentRes, publishComment);
        publishComment.setOperatorId(operatorId);
        publishComment.setUserId(SecurityUtils.getUserId());
        publishComment.setType(0);

        publishCommentMapper.insert(publishComment);

        return AjaxResult.success("评论成功");
    }


    /**
     * 构建CommentVo对象
     *
     * @param Comment 父评论对象
     * @return CommentVo对象
     */
    @NotNull
    private PublicModelCommentVo getModelCommentVo(PublishComment Comment) {
        Long userId = Comment.getUserId();
        SysUser sysUser = sysUserService.selectUserById(userId);

        // 构建CommentVo对象
        PublicModelCommentVo CommentVo = new PublicModelCommentVo();
        CommentVo.setUserId(userId);
        CommentVo.setUserName(sysUser.getUserName());
        CommentVo.setUserAvatar(sysUser.getAvatar());
        CommentVo.setCommentId(Comment.getId());
        CommentVo.setContent(Comment.getContent());
        // 查询子评论
        CommentVo.setContentList(getContentList(Comment.getId()));
        CommentVo.setLikeNum(Comment.getLikeNum());
        CommentVo.setCreateTime(Comment.getCreateTime());
        return CommentVo;
    }


    /**
     * 递归查询子评论
     *
     * @param CommentId 父评论id
     * @return List<ModelCommentVo>
     */
    private List<PublicModelCommentVo> getContentList(Long CommentId) {
        List<PublicModelCommentVo> CommentVoList = new ArrayList<>();
        if (Objects.isNull(CommentId)) {
            return CommentVoList;
        }

        // 查询子评论
        LambdaQueryWrapper<PublishComment> lqw = new LambdaQueryWrapper<PublishComment>()
                .eq(PublishComment::getParentId, CommentId)
                .orderByDesc(PublishComment::getCreateTime);

        List<PublishComment> CommentList = publishCommentMapper.selectList(lqw);

        for (PublishComment Comment : CommentList) {
            Long userId = Comment.getUserId();
            SysUser sysUser = sysUserService.selectUserById(userId);
            PublicModelCommentVo CommentVo = new PublicModelCommentVo();
            CommentVo.setUserId(userId);
            CommentVo.setUserName(sysUser.getUserName());
            CommentVo.setUserAvatar(sysUser.getAvatar());
            CommentVo.setCommentId(Comment.getId());
            CommentVo.setContent(Comment.getContent());
            CommentVo.setLikeNum(Comment.getLikeNum());
            CommentVo.setCreateTime(Comment.getCreateTime());


            CommentVoList.add(CommentVo);
        }
        return CommentVoList;
    }
}
